package sun.print;

import sun.security.action.GetPropertyAction;
import java.awt.Graphics2D;
import sun.awt.image.ByteInterleavedRaster;
import java.awt.Color;
import java.awt.geom.Point2D;
import java.awt.print.PrinterAbortException;
import java.util.Locale;
import java.io.IOException;
import javax.print.attribute.standard.PrinterIsAcceptingJobs;
import javax.print.DocPrintJob;
import javax.print.PrintException;
import javax.print.attribute.standard.RequestingUserName;
import javax.print.attribute.standard.JobSheets;
import javax.print.attribute.standard.SheetCollate;
import javax.print.attribute.standard.Fidelity;
import javax.print.attribute.Attribute;
import java.io.File;
import javax.print.attribute.standard.Destination;
import javax.print.attribute.standard.JobName;
import javax.print.attribute.standard.Copies;
import javax.print.StreamPrintServiceFactory;
import javax.print.ServiceUI;
import java.awt.Rectangle;
import java.awt.Frame;
import java.awt.HeadlessException;
import javax.print.attribute.standard.DialogTypeSelection;
import java.security.AccessController;
import java.awt.GraphicsConfiguration;
import java.security.PrivilegedAction;
import java.awt.GraphicsEnvironment;
import javax.print.attribute.HashPrintRequestAttributeSet;
import javax.print.attribute.standard.MediaSizeName;
import javax.print.attribute.standard.MediaSize;
import javax.print.attribute.standard.MediaPrintableArea;
import javax.print.attribute.standard.Media;
import javax.print.attribute.standard.OrientationRequested;
import javax.print.attribute.standard.PrinterStateReason;
import javax.print.attribute.standard.PrinterStateReasons;
import javax.print.attribute.standard.PrinterState;
import javax.print.StreamPrintService;
import javax.print.DocFlavor;
import javax.print.PrintServiceLookup;
import java.awt.geom.Rectangle2D;
import java.awt.Shape;
import java.awt.print.PrinterException;
import java.awt.print.Printable;
import java.awt.print.PageFormat;
import java.awt.print.Book;
import java.awt.geom.AffineTransform;
import javax.print.PrintService;
import javax.print.attribute.PrintRequestAttributeSet;
import javax.print.attribute.standard.Sides;
import javax.print.attribute.standard.PageRanges;
import java.util.ArrayList;
import java.io.FilePermission;
import java.awt.print.Pageable;
import java.awt.print.Paper;
import java.awt.image.BufferedImage;
import java.awt.print.PrinterJob;

public abstract class RasterPrinterJob extends PrinterJob
{
    protected static final int PRINTER = 0;
    protected static final int FILE = 1;
    protected static final int STREAM = 2;
    protected static final int MAX_UNKNOWN_PAGES = 9999;
    protected static final int PD_ALLPAGES = 0;
    protected static final int PD_SELECTION = 1;
    protected static final int PD_PAGENUMS = 2;
    protected static final int PD_NOSELECTION = 4;
    private static final int MAX_BAND_SIZE = 4194304;
    private static final float DPI = 72.0f;
    private static final String FORCE_PIPE_PROP = "sun.java2d.print.pipeline";
    private static final String FORCE_RASTER = "raster";
    private static final String FORCE_PDL = "pdl";
    private static final String SHAPE_TEXT_PROP = "sun.java2d.print.shapetext";
    public static boolean forcePDL;
    public static boolean forceRaster;
    public static boolean shapeTextProp;
    private int cachedBandWidth;
    private int cachedBandHeight;
    private BufferedImage cachedBand;
    private int mNumCopies;
    private boolean mCollate;
    private int mFirstPage;
    private int mLastPage;
    private Paper previousPaper;
    protected Pageable mDocument;
    private String mDocName;
    protected boolean performingPrinting;
    protected boolean userCancelled;
    private FilePermission printToFilePermission;
    private ArrayList redrawList;
    private int copiesAttr;
    private String jobNameAttr;
    private String userNameAttr;
    private PageRanges pageRangesAttr;
    protected Sides sidesAttr;
    protected String destinationAttr;
    protected boolean noJobSheet;
    protected int mDestType;
    protected String mDestination;
    protected boolean collateAttReq;
    protected boolean landscapeRotates270;
    protected PrintRequestAttributeSet attributes;
    protected PrintService myService;
    public static boolean debugPrint;
    private int deviceWidth;
    private int deviceHeight;
    private AffineTransform defaultDeviceTransform;
    private PrinterGraphicsConfig pgConfig;
    private DialogOnTop onTop;
    private long parentWindowID;
    
    public RasterPrinterJob() {
        this.cachedBandWidth = 0;
        this.cachedBandHeight = 0;
        this.cachedBand = null;
        this.mNumCopies = 1;
        this.mCollate = false;
        this.mFirstPage = -1;
        this.mLastPage = -1;
        this.mDocument = new Book();
        this.mDocName = "Java Printing";
        this.performingPrinting = false;
        this.userCancelled = false;
        this.redrawList = new ArrayList();
        this.noJobSheet = false;
        this.mDestType = 1;
        this.mDestination = "";
        this.collateAttReq = false;
        this.landscapeRotates270 = false;
        this.attributes = null;
        this.onTop = null;
        this.parentWindowID = 0L;
    }
    
    protected abstract double getXRes();
    
    protected abstract double getYRes();
    
    protected abstract double getPhysicalPrintableX(Paper p0);
    
    protected abstract double getPhysicalPrintableY(Paper p0);
    
    protected abstract double getPhysicalPrintableWidth(Paper p0);
    
    protected abstract double getPhysicalPrintableHeight(Paper p0);
    
    protected abstract double getPhysicalPageWidth(Paper p0);
    
    protected abstract double getPhysicalPageHeight(Paper p0);
    
    protected abstract void startPage(PageFormat p0, Printable p1, int p2, boolean p3) throws PrinterException;
    
    protected abstract void endPage(PageFormat p0, Printable p1, int p2) throws PrinterException;
    
    protected abstract void printBand(byte[] p0, int p1, int p2, int p3, int p4) throws PrinterException;
    
    public void saveState(AffineTransform theTransform, Shape theClip, Rectangle2D region, double sx, double sy) {
        RasterPrinterJob.GraphicsState e = new RasterPrinterJob.GraphicsState();
        e.theTransform = theTransform;
        e.theClip = theClip;
        e.region = region;
        e.sx = sx;
        e.sy = sy;
        this.redrawList.add(e);
    }
    
    protected static PrintService lookupDefaultPrintService() {
        PrintService lookupDefaultPrintService = PrintServiceLookup.lookupDefaultPrintService();
        if (lookupDefaultPrintService != null && lookupDefaultPrintService.isDocFlavorSupported(DocFlavor.SERVICE_FORMATTED.PAGEABLE) && lookupDefaultPrintService.isDocFlavorSupported(DocFlavor.SERVICE_FORMATTED.PRINTABLE)) {
            return lookupDefaultPrintService;
        }
        PrintService[] lookupPrintServices = PrintServiceLookup.lookupPrintServices(DocFlavor.SERVICE_FORMATTED.PAGEABLE, null);
        if (lookupPrintServices.length > 0) {
            return lookupPrintServices[0];
        }
        return null;
    }
    
    @Override
    public PrintService getPrintService() {
        if (this.myService == null) {
            PrintService lookupDefaultPrintService = PrintServiceLookup.lookupDefaultPrintService();
            if (lookupDefaultPrintService != null && lookupDefaultPrintService.isDocFlavorSupported(DocFlavor.SERVICE_FORMATTED.PAGEABLE)) {
                try {
                    this.setPrintService(lookupDefaultPrintService);
                    this.myService = lookupDefaultPrintService;
                }
                catch (PrinterException ex) {}
            }
            if (this.myService == null) {
                PrintService[] lookupPrintServices = PrintServiceLookup.lookupPrintServices(DocFlavor.SERVICE_FORMATTED.PAGEABLE, null);
                if (lookupPrintServices.length > 0) {
                    try {
                        this.setPrintService(lookupPrintServices[0]);
                        this.myService = lookupPrintServices[0];
                    }
                    catch (PrinterException ex2) {}
                }
            }
        }
        return this.myService;
    }
    
    @Override
    public void setPrintService(PrintService printService) throws PrinterException {
        if (printService == null) {
            throw new PrinterException("Service cannot be null");
        }
        if (!(printService instanceof StreamPrintService) && printService.getName() == null) {
            throw new PrinterException("Null PrintService name.");
        }
        if ((PrinterState)printService.getAttribute(PrinterState.class) == PrinterState.STOPPED) {
            PrinterStateReasons printerStateReasons = (PrinterStateReasons)printService.getAttribute(PrinterStateReasons.class);
            if (printerStateReasons != null && printerStateReasons.containsKey(PrinterStateReason.SHUTDOWN)) {
                throw new PrinterException("PrintService is no longer available.");
            }
        }
        if (printService.isDocFlavorSupported(DocFlavor.SERVICE_FORMATTED.PAGEABLE) && printService.isDocFlavorSupported(DocFlavor.SERVICE_FORMATTED.PRINTABLE)) {
            this.myService = printService;
            return;
        }
        throw new PrinterException("Not a 2D print service: " + printService);
    }
    
    private PageFormat attributeToPageFormat(PrintService printService, PrintRequestAttributeSet set) {
        PageFormat defaultPage = super.defaultPage();
        if (printService == null) {
            return defaultPage;
        }
        OrientationRequested orientationRequested = (OrientationRequested)set.get(OrientationRequested.class);
        if (orientationRequested == null) {
            orientationRequested = (OrientationRequested)printService.getDefaultAttributeValue(OrientationRequested.class);
        }
        if (orientationRequested == OrientationRequested.REVERSE_LANDSCAPE) {
            defaultPage.setOrientation(2);
        }
        else if (orientationRequested == OrientationRequested.LANDSCAPE) {
            defaultPage.setOrientation(0);
        }
        else {
            defaultPage.setOrientation(1);
        }
        MediaSize mediaSize = this.getMediaSize((Media)set.get(Media.class), printService, defaultPage);
        Paper paper = new Paper();
        float[] size = mediaSize.getSize(1);
        double rint = Math.rint((double)size[0] * 72.0 / 25400.0);
        double rint2 = Math.rint((double)size[1] * 72.0 / 25400.0);
        paper.setSize(rint, rint2);
        MediaPrintableArea defaultPrintableArea = (MediaPrintableArea)set.get(MediaPrintableArea.class);
        if (defaultPrintableArea == null) {
            defaultPrintableArea = this.getDefaultPrintableArea(defaultPage, rint, rint2);
        }
        paper.setImageableArea(Math.rint((double)(defaultPrintableArea.getX(25400) * 72.0f)), Math.rint((double)(defaultPrintableArea.getY(25400) * 72.0f)), Math.rint((double)(defaultPrintableArea.getWidth(25400) * 72.0f)), Math.rint((double)(defaultPrintableArea.getHeight(25400) * 72.0f)));
        defaultPage.setPaper(paper);
        return defaultPage;
    }
    
    protected MediaSize getMediaSize(Media na_LETTER, PrintService printService, PageFormat pageFormat) {
        if (na_LETTER == null) {
            na_LETTER = (Media)printService.getDefaultAttributeValue(Media.class);
        }
        if (!(na_LETTER instanceof MediaSizeName)) {
            na_LETTER = MediaSizeName.NA_LETTER;
        }
        MediaSize mediaSizeForName = MediaSize.getMediaSizeForName((MediaSizeName)na_LETTER);
        return (mediaSizeForName != null) ? mediaSizeForName : MediaSize.NA.LETTER;
    }
    
    protected MediaPrintableArea getDefaultPrintableArea(PageFormat pageFormat, double n, double n2) {
        double n3;
        double n4;
        if (n >= 432.0) {
            n3 = 72.0;
            n4 = n - 144.0;
        }
        else {
            n3 = n / 6.0;
            n4 = n * 0.75;
        }
        double n5;
        double n6;
        if (n2 >= 432.0) {
            n5 = 72.0;
            n6 = n2 - 144.0;
        }
        else {
            n5 = n2 / 6.0;
            n6 = n2 * 0.75;
        }
        return new MediaPrintableArea((float)(n3 / 72.0), (float)(n5 / 72.0), (float)(n4 / 72.0), (float)(n6 / 72.0), 25400);
    }
    
    protected void updatePageAttributes(PrintService printService, PageFormat pageFormat) {
        if (this.attributes == null) {
            this.attributes = new HashPrintRequestAttributeSet();
        }
        this.updateAttributesWithPageFormat(printService, pageFormat, this.attributes);
    }
    
    protected void updateAttributesWithPageFormat(PrintService printService, PageFormat pageFormat, PrintRequestAttributeSet set) {
        if (printService == null || pageFormat == null || set == null) {
            return;
        }
        float n = (float)Math.rint(pageFormat.getPaper().getWidth() * 25400.0 / 72.0) / 25400.0f;
        float n2 = (float)Math.rint(pageFormat.getPaper().getHeight() * 25400.0 / 72.0) / 25400.0f;
        Media[] array = (Media[])(Media[])printService.getSupportedAttributeValues(Media.class, null, null);
        Media media = null;
        try {
            media = CustomMediaSizeName.findMedia(array, n, n2, 25400);
        }
        catch (IllegalArgumentException ex) {}
        if (media == null || !printService.isAttributeValueSupported(media, null, null)) {
            media = (Media)printService.getDefaultAttributeValue(Media.class);
        }
        OrientationRequested orientationRequested;
        switch (pageFormat.getOrientation()) {
            case 0: {
                orientationRequested = OrientationRequested.LANDSCAPE;
                break;
            }
            case 2: {
                orientationRequested = OrientationRequested.REVERSE_LANDSCAPE;
                break;
            }
            default: {
                orientationRequested = OrientationRequested.PORTRAIT;
                break;
            }
        }
        if (media != null) {
            set.add(media);
        }
        set.add(orientationRequested);
        float x = (float)(pageFormat.getPaper().getImageableX() / 72.0);
        float w = (float)(pageFormat.getPaper().getImageableWidth() / 72.0);
        float y = (float)(pageFormat.getPaper().getImageableY() / 72.0);
        float h = (float)(pageFormat.getPaper().getImageableHeight() / 72.0);
        if (x < 0.0f) {
            x = 0.0f;
        }
        if (y < 0.0f) {
            y = 0.0f;
        }
        try {
            set.add(new MediaPrintableArea(x, y, w, h, 25400));
        }
        catch (IllegalArgumentException ex2) {}
        return;
    }
    
    @Override
    public PageFormat pageDialog(PageFormat pageFormat) throws HeadlessException {
        if (GraphicsEnvironment.isHeadless()) {
            throw new HeadlessException();
        }
        PrintService printService = (PrintService)AccessController.doPrivileged(new PrivilegedAction() {
            
            @Override
            public Object run() {
                PrintService printService = RasterPrinterJob.this.getPrintService();
                if (printService == null) {
                    ServiceDialog.showNoPrintService(gc);
                    return null;
                }
                return printService;
            }
        });
        if (printService == null) {
            return pageFormat;
        }
        this.updatePageAttributes(printService, pageFormat);
        PageFormat pageFormat2;
        if ((DialogTypeSelection)this.attributes.get(DialogTypeSelection.class) == DialogTypeSelection.NATIVE) {
            this.attributes.remove(DialogTypeSelection.class);
            pageFormat2 = this.pageDialog(this.attributes);
            this.attributes.add(DialogTypeSelection.NATIVE);
        }
        else {
            pageFormat2 = this.pageDialog(this.attributes);
        }
        if (pageFormat2 == null) {
            return pageFormat;
        }
        return pageFormat2;
    }
    
    @Override
    public PageFormat pageDialog(PrintRequestAttributeSet parentWindowID) throws HeadlessException {
        if (GraphicsEnvironment.isHeadless()) {
            throw new HeadlessException();
        }
        if ((DialogTypeSelection)parentWindowID.get(DialogTypeSelection.class) == DialogTypeSelection.NATIVE) {
            PrintService printService2 = this.getPrintService();
            PageFormat attributeToPageFormat = this.attributeToPageFormat(printService2, parentWindowID);
            this.setParentWindowID(parentWindowID);
            PageFormat pageDialog = this.pageDialog(attributeToPageFormat);
            this.clearParentWindowID();
            if (pageDialog == attributeToPageFormat) {
                return null;
            }
            this.updateAttributesWithPageFormat(printService2, pageDialog, parentWindowID);
            return pageDialog;
        }
        else {
            final GraphicsConfiguration defaultConfiguration = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().getDefaultConfiguration();
            Rectangle bounds = defaultConfiguration.getBounds();
            int n = bounds.x + bounds.width / 3;
            int n2 = bounds.y + bounds.height / 3;
            PrintService printService = (PrintService)AccessController.doPrivileged(new PrivilegedAction() {
                @Override
                public Object run() {
                    PrintService printService = RasterPrinterJob.this.getPrintService();
                    if (printService == null) {
                        ServiceDialog.showNoPrintService(defaultConfiguration);
                        return null;
                    }
                    return printService;
                }
            });
            if (printService == null) {
                return null;
            }
            if (this.onTop != null) {
                parentWindowID.add(this.onTop);
            }
            ServiceDialog serviceDialog = new ServiceDialog(defaultConfiguration, n, n2, printService, DocFlavor.SERVICE_FORMATTED.PAGEABLE, parentWindowID, (Frame)null);
            serviceDialog.show();
            if (serviceDialog.getStatus() == 1) {
                PrintRequestAttributeSet attributes = serviceDialog.getAttributes();
                Class<Object> clazz = SunAlternateMedia.class;
                if (parentWindowID.containsKey(clazz) && !attributes.containsKey(clazz)) {
                    parentWindowID.remove(clazz);
                }
                parentWindowID.addAll(attributes);
                return this.attributeToPageFormat(printService, parentWindowID);
            }
            else {
                return null;
            }
        }
    }
    
    protected PageFormat getPageFormatFromAttributes() {
        Pageable pageable;
        if (this.attributes == null || this.attributes.isEmpty() || !((pageable = this.getPageable()) instanceof OpenBook)) {
            return null;
        }
        PageFormat attributeToPageFormat = this.attributeToPageFormat(this.getPrintService(), this.attributes);
        PageFormat pageFormat;
        if ((pageFormat = pageable.getPageFormat(0)) != null) {
            if (this.attributes.get(OrientationRequested.class) == null) {
                attributeToPageFormat.setOrientation(pageFormat.getOrientation());
            }
            Paper paper = attributeToPageFormat.getPaper();
            Paper paper2 = pageFormat.getPaper();
            boolean n = false;
            if (this.attributes.get(MediaSizeName.class) == null) {
                paper.setSize(paper2.getWidth(), paper2.getHeight());
                n = true;
            }
            if (this.attributes.get(MediaPrintableArea.class) == null) {
                paper.setImageableArea(paper2.getImageableX(), paper2.getImageableY(), paper2.getImageableWidth(), paper2.getImageableHeight());
                n = true;
            }
            if (!(n == 0)) {
                attributeToPageFormat.setPaper(paper);
            }
        }
        return attributeToPageFormat;
    }
    
    @Override
    public boolean printDialog(PrintRequestAttributeSet set) throws HeadlessException {
        if (GraphicsEnvironment.isHeadless()) {
            throw new HeadlessException();
        }
        if ((DialogTypeSelection)set.get(DialogTypeSelection.class) == DialogTypeSelection.NATIVE) {
            this.attributes = set;
            try {
                this.debug_println("calling setAttributes in printDialog");
                this.setAttributes(set);
            }
            catch (PrinterException ex3) {}
            this.setParentWindowID(set);
            int printDialog = this.printDialog();
            this.clearParentWindowID();
            this.attributes = set;
            return printDialog;
        }
        final GraphicsConfiguration defaultConfiguration = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().getDefaultConfiguration();
        PrintService defaultService = (PrintService)AccessController.doPrivileged(new PrivilegedAction() {
            @Override
            public Object run() {
                PrintService printService = RasterPrinterJob.this.getPrintService();
                if (printService == null) {
                    ServiceDialog.showNoPrintService(defaultConfiguration);
                    return null;
                }
                return printService;
            }
        });
        if (defaultService == null) {
            return false;
        }
        StreamPrintService[] array;
        if ((defaultService instanceof StreamPrintService)) {
            StreamPrintServiceFactory[] lookupStreamPrintServices = PrinterJob.lookupStreamPrintServices(null);
            array = new StreamPrintService[lookupStreamPrintServices.length];
            for (int i = 0; i < lookupStreamPrintServices.length; ++i) {
                array[i] = lookupStreamPrintServices[i].getPrintService(null);
            }
        }
        else {
            array = (PrintService[])(PrintService[])AccessController.doPrivileged(new PrivilegedAction() {
                @Override
                public Object run() {
                    return PrinterJob.lookupPrintServices();
                }
            });
            if (array == null || array.length == 0) {
                array = new PrintService[] { defaultService };
            }
        }
        Rectangle bounds = defaultConfiguration.getBounds();
        int n = bounds.x + bounds.width / 3;
        int n2 = bounds.y + bounds.height / 3;
        set.add(new PrinterJobWrapper(this));
        PrintService printService;
        try {
            printService = ServiceUI.printDialog(defaultConfiguration, n, n2, array, defaultService, DocFlavor.SERVICE_FORMATTED.PAGEABLE, set);
        }
        catch (IllegalArgumentException ex) {
            printService = ServiceUI.printDialog(defaultConfiguration, n, n2, array, array[0], DocFlavor.SERVICE_FORMATTED.PAGEABLE, set);
        }
        set.remove(PrinterJobWrapper.class);
        if (printService == null) {
            return false;
        }
        if (!defaultService.equals(printService)) {
            try {
                this.setPrintService(printService);
            }
            catch (PrinterException ex2) {
                this.myService = printService;
            }
        }
        return true;
    }
    
    @Override
    public boolean printDialog() throws HeadlessException {
        if (GraphicsEnvironment.isHeadless()) {
            throw new HeadlessException();
        }
        HashPrintRequestAttributeSet set = new HashPrintRequestAttributeSet();
        set.add(new Copies(this.getCopies()));
        set.add(new JobName(this.getJobName(), null));
        int printDialog = this.printDialog(set);
        if (!(printDialog == 0)) {
            JobName jobName = (JobName)set.get(JobName.class);
            if (jobName != null) {
                this.setJobName(jobName.getValue());
            }
            Copies copies = (Copies)set.get(Copies.class);
            if (copies != null) {
                this.setCopies(copies.getValue());
            }
            Destination destination = (Destination)set.get(Destination.class);
            if (destination != null) {
                try {
                    this.mDestType = 1;
                    this.mDestination = new File(destination.getURI()).getPath();
                }
                catch (Exception ex) {
                    this.mDestination = "out.prn";
                    PrintService printService2 = this.getPrintService();
                    if (printService2 != null) {
                        Destination destination2 = (Destination)printService2.getDefaultAttributeValue(Destination.class);
                        if (destination2 != null) {
                            this.mDestination = new File(destination2.getURI()).getPath();
                        }
                    }
                }
            }
            else {
                this.mDestType = 0;
                PrintService printService = this.getPrintService();
                if (printService != null) {
                    this.mDestination = printService.getName();
                }
            }
        }
        return printDialog;
    }
    
    @Override
    public void setPrintable(Printable printable) {
        this.setPageable(new OpenBook(this.defaultPage(new PageFormat()), printable));
    }
    
    @Override
    public void setPrintable(Printable printable, PageFormat pageFormat) {
        this.setPageable(new OpenBook(pageFormat, printable));
        this.updatePageAttributes(this.getPrintService(), pageFormat);
    }
    
    @Override
    public void setPageable(Pageable mDocument) throws NullPointerException {
        if (mDocument != null) {
            this.mDocument = mDocument;
            return;
        }
        throw new NullPointerException();
    }
    
    protected void initPrinter() {
    }
    
    protected boolean isSupportedValue(Attribute attribute, PrintRequestAttributeSet set) {
        PrintService printService = this.getPrintService();
        return attribute != null && printService != null && printService.isAttributeValueSupported(attribute, DocFlavor.SERVICE_FORMATTED.PAGEABLE, set);
    }
    
    protected void setAttributes(PrintRequestAttributeSet attributes) throws PrinterException {
        this.setCollated(false);
        this.sidesAttr = null;
        this.pageRangesAttr = null;
        this.copiesAttr = 0;
        this.jobNameAttr = null;
        this.userNameAttr = null;
        this.destinationAttr = null;
        this.collateAttReq = false;
        PrintService printService = this.getPrintService();
        if (attributes == null || printService == null) {
            return;
        }
        boolean n = false;
        Fidelity fidelity = (Fidelity)attributes.get(Fidelity.class);
        if (fidelity != null && fidelity == Fidelity.FIDELITY_TRUE) {
            n = true;
        }
        if (n && printService.getUnsupportedAttributes(DocFlavor.SERVICE_FORMATTED.PAGEABLE, attributes) != null) {
            throw new PrinterException("Fidelity cannot be satisfied");
        }
        SheetCollate sheetCollate = (SheetCollate)attributes.get(SheetCollate.class);
        if (this.isSupportedValue(sheetCollate, attributes)) {
            this.setCollated(sheetCollate == SheetCollate.COLLATED);
        }
        this.sidesAttr = (Sides)attributes.get(Sides.class);
        if (!this.isSupportedValue(this.sidesAttr, attributes)) {
            this.sidesAttr = Sides.ONE_SIDED;
        }
        this.pageRangesAttr = (PageRanges)attributes.get(PageRanges.class);
        if (!this.isSupportedValue(this.pageRangesAttr, attributes)) {
            this.pageRangesAttr = null;
        }
        else if ((SunPageSelection)attributes.get(SunPageSelection.class) == SunPageSelection.RANGE) {
            int[][] members = this.pageRangesAttr.getMembers();
            this.setPageRange(members[0][0] - 1, members[0][1] - 1);
        }
        else {
            this.setPageRange(-1, -1);
        }
        Copies copies = (Copies)attributes.get(Copies.class);
        if (this.isSupportedValue(copies, attributes) || (n == 0 && copies != null)) {
            this.setCopies(this.copiesAttr = copies.getValue());
        }
        else {
            this.copiesAttr = this.getCopies();
        }
        Destination destination = (Destination)attributes.get(Destination.class);
        if (this.isSupportedValue(destination, attributes)) {
            try {
                this.destinationAttr = "" + new File(destination.getURI().getSchemeSpecificPart());
            }
            catch (Exception ex) {
                Destination destination2 = (Destination)printService.getDefaultAttributeValue(Destination.class);
                if (destination2 != null) {
                    this.destinationAttr = "" + new File(destination2.getURI().getSchemeSpecificPart());
                }
            }
        }
        JobSheets jobSheets = (JobSheets)attributes.get(JobSheets.class);
        if (jobSheets != null) {
            this.noJobSheet = (jobSheets == JobSheets.NONE);
        }
        JobName jobName = (JobName)attributes.get(JobName.class);
        if (this.isSupportedValue(jobName, attributes) || (n == 0 && jobName != null)) {
            this.setJobName(this.jobNameAttr = jobName.getValue());
        }
        else {
            this.jobNameAttr = this.getJobName();
        }
        RequestingUserName requestingUserName = (RequestingUserName)attributes.get(RequestingUserName.class);
        if (this.isSupportedValue(requestingUserName, attributes) || (n == 0 && requestingUserName != null)) {
            this.userNameAttr = requestingUserName.getValue();
        }
        else {
            try {
                this.userNameAttr = this.getUserName();
            }
            catch (SecurityException ex2) {
                this.userNameAttr = "";
            }
        }
        Media media = (Media)attributes.get(Media.class);
        OrientationRequested orientationRequested = (OrientationRequested)attributes.get(OrientationRequested.class);
        MediaPrintableArea mediaPrintableArea = (MediaPrintableArea)attributes.get(MediaPrintableArea.class);
        if ((orientationRequested != null || media != null || mediaPrintableArea != null) && this.getPageable() instanceof OpenBook) {
            Pageable pageable = this.getPageable();
            Printable printable = pageable.getPrintable(0);
            PageFormat pageFormat = (PageFormat)pageable.getPageFormat(0).clone();
            Paper paper = pageFormat.getPaper();
            if (mediaPrintableArea == null && media != null && printService.isAttributeCategorySupported(MediaPrintableArea.class)) {
                Object supportedAttributeValues = printService.getSupportedAttributeValues(MediaPrintableArea.class, null, attributes);
                if (supportedAttributeValues instanceof MediaPrintableArea[] && ((MediaPrintableArea[])(MediaPrintableArea[])supportedAttributeValues).length > 0) {
                    mediaPrintableArea = ((MediaPrintableArea[])(MediaPrintableArea[])supportedAttributeValues)[0];
                }
            }
            if (this.isSupportedValue(orientationRequested, attributes) || (n == 0 && orientationRequested != null)) {
                int orientation;
                if (orientationRequested.equals(OrientationRequested.REVERSE_LANDSCAPE)) {
                    orientation = 2;
                }
                else if (orientationRequested.equals(OrientationRequested.LANDSCAPE)) {
                    orientation = 0;
                }
                else {
                    orientation = 1;
                }
                pageFormat.setOrientation(orientation);
            }
            if ((this.isSupportedValue(media, attributes) || (n == 0 && media != null)) && media instanceof MediaSizeName) {
                MediaSize mediaSizeForName = MediaSize.getMediaSizeForName((MediaSizeName)media);
                if (mediaSizeForName != null) {
                    float n2 = mediaSizeForName.getX(25400) * 72.0f;
                    float n3 = mediaSizeForName.getY(25400) * 72.0f;
                    paper.setSize((double)n2, (double)n3);
                    if (mediaPrintableArea == null) {
                        paper.setImageableArea(72.0, 72.0, (double)n2 - 144.0, (double)n3 - 144.0);
                    }
                }
            }
            if (this.isSupportedValue(mediaPrintableArea, attributes) || (n == 0 && mediaPrintableArea != null)) {
                float[] printableArea = mediaPrintableArea.getPrintableArea(25400);
                for (int i = 0; i < printableArea.length; ++i) {
                    printableArea[i] *= 72.0f;
                }
                paper.setImageableArea((double)printableArea[0], (double)printableArea[1], (double)printableArea[2], (double)printableArea[3]);
            }
            pageFormat.setPaper(paper);
            this.setPrintable(printable, this.validatePage(pageFormat));
        }
        else {
            this.attributes = attributes;
        }
        return;
    }
    
    protected void spoolToService(PrintService printService, PrintRequestAttributeSet set) throws PrinterException {
        if (printService == null) {
            throw new PrinterException("No print service found.");
        }
        DocPrintJob printJob = printService.createPrintJob();
        PageableDoc pageableDoc = new PageableDoc(this.getPageable());
        if (set == null) {
            set = new HashPrintRequestAttributeSet();
        }
        try {
            printJob.print(pageableDoc, set);
        }
        catch (PrintException ex) {
            throw new PrinterException(ex.toString());
        }
        return;
    }
    
    @Override
    public void print() throws PrinterException {
        this.print(this.attributes);
    }
    
    protected void debug_println(String str) {
        if (RasterPrinterJob.debugPrint) {
            System.out.println("RasterPrinterJob " + str + " " + this);
        }
    }
    
    @Override
    public void print(PrintRequestAttributeSet attributes) throws PrinterException {
        PrintService printService = this.getPrintService();
        this.debug_println("psvc = " + printService);
        if (printService == null) {
            throw new PrinterException("No print service found.");
        }
        if ((PrinterState)printService.getAttribute(PrinterState.class) == PrinterState.STOPPED) {
            PrinterStateReasons printerStateReasons = (PrinterStateReasons)printService.getAttribute(PrinterStateReasons.class);
            if (printerStateReasons != null && printerStateReasons.containsKey(PrinterStateReason.SHUTDOWN)) {
                throw new PrinterException("PrintService is no longer available.");
            }
        }
        if ((PrinterIsAcceptingJobs)printService.getAttribute(PrinterIsAcceptingJobs.class) == PrinterIsAcceptingJobs.NOT_ACCEPTING_JOBS) {
            throw new PrinterException("Printer is not accepting job.");
        }
        if (printService instanceof SunPrinterJobService && ((SunPrinterJobService)printService).usesClass(this.getClass())) {
            this.setAttributes(attributes);
            if (this.destinationAttr != null) {
                this.validateDestination(this.destinationAttr);
            }
            this.initPrinter();
            int collatedCopies = this.getCollatedCopies();
            int noncollatedCopies = this.getNoncollatedCopies();
            this.debug_println("getCollatedCopies()  " + collatedCopies + " getNoncollatedCopies() " + noncollatedCopies);
            if (this.mDocument.getNumberOfPages() == 0) {
                return;
            }
            int firstPage = this.getFirstPage();
            int lastPage = this.getLastPage();
            if (lastPage == -1 && this.mDocument.getNumberOfPages() != -1) {
                lastPage = this.mDocument.getNumberOfPages() - 1;
            }
            try {
                synchronized (this) {
                    this.performingPrinting = true;
                    this.userCancelled = false;
                }
                this.startDoc();
                if (this.isCancelled()) {
                    this.cancelDoc();
                }
                boolean b = true;
                if (attributes != null) {
                    SunPageSelection sunPageSelection = (SunPageSelection)attributes.get(SunPageSelection.class);
                    if (sunPageSelection != null && sunPageSelection != SunPageSelection.RANGE) {
                        b = false;
                    }
                }
                this.debug_println("after startDoc rangeSelected? " + (boolean)b + " numNonCollatedCopies " + noncollatedCopies);
                for (int i = 0; i < collatedCopies; ++i) {
                    for (int printPage = 0; (firstPage <= lastPage || lastPage == -1) && printPage == 0; ++firstPage) {
                        if (this.pageRangesAttr != null && b) {
                            int next = this.pageRangesAttr.next(firstPage);
                            if (next == -1) {
                                break;
                            }
                            if (next != firstPage + 1) {
                                continue;
                            }
                        }
                        for (int n = 0; n < noncollatedCopies && printPage == 0; printPage = this.printPage(this.mDocument, firstPage), ++n) {
                            if (this.isCancelled()) {
                                this.cancelDoc();
                            }
                            this.debug_println("printPage " + firstPage);
                        }
                    }
                }
                if (this.isCancelled()) {
                    this.cancelDoc();
                }
            }
            finally {
                this.previousPaper = null;
                synchronized (this) {
                    if (this.performingPrinting) {
                        this.endDoc();
                    }
                    this.performingPrinting = false;
                    this.notify();
                }
            }
            return;
        }
        else {
            this.spoolToService(printService, attributes);
            return;
        }
    }
    
    protected void validateDestination(String str) throws PrinterException {
        if (str == null) {
            return;
        }
        File file = new File(str);
        try {
            if (file.createNewFile()) {
                file.delete();
            }
        }
        catch (IOException ex) {
            throw new PrinterException("Cannot write to file:" + str);
        }
        catch (SecurityException ex2) {}
        File parentFile = file.getParentFile();
        if ((file.exists() && (!file.isFile() || !file.canWrite())) || (parentFile != null && (!parentFile.exists() || (parentFile.exists() && !parentFile.canWrite())))) {
            throw new PrinterException("Cannot write to file:" + str);
        }
        return;
    }
    
    protected void validatePaper(Paper paper, Paper paper2) {
        if (paper == null || paper2 == null) {
            return;
        }
        double width = paper.getWidth();
        double height = paper.getHeight();
        double imageableX = paper.getImageableX();
        double imageableY = paper.getImageableY();
        double imageableWidth = paper.getImageableWidth();
        double imageableHeight = paper.getImageableHeight();
        Paper paper3 = new Paper();
        double width2 = (width > 0.0) ? width : paper3.getWidth();
        double height2 = (height > 0.0) ? height : paper3.getHeight();
        double x = (imageableX > 0.0) ? imageableX : paper3.getImageableX();
        double y = (imageableY > 0.0) ? imageableY : paper3.getImageableY();
        double width3 = (imageableWidth > 0.0) ? imageableWidth : paper3.getImageableWidth();
        double height3 = (imageableHeight > 0.0) ? imageableHeight : paper3.getImageableHeight();
        if (width3 > width2) {
            width3 = width2;
        }
        if (height3 > height2) {
            height3 = height2;
        }
        if (x + width3 > width2) {
            x = width2 - width3;
        }
        if (y + height3 > height2) {
            y = height2 - height3;
        }
        paper2.setSize(width2, height2);
        paper2.setImageableArea(x, y, width3, height3);
        return;
    }
    
    @Override
    public PageFormat defaultPage(PageFormat pageFormat) {
        PageFormat pageFormat2 = (PageFormat)pageFormat.clone();
        pageFormat2.setOrientation(1);
        Paper paper = new Paper();
        double n = 72.0;
        PrintService printService = this.getPrintService();
        if (printService != null) {
            Media media = (Media)printService.getDefaultAttributeValue(Media.class);
            MediaSize mediaSizeForName;
            if (media instanceof MediaSizeName && (mediaSizeForName = MediaSize.getMediaSizeForName((MediaSizeName)media)) != null) {
                double width = (double)mediaSizeForName.getX(25400) * n;
                double height = (double)mediaSizeForName.getY(25400) * n;
                paper.setSize(width, height);
                paper.setImageableArea(n, n, width - 2.0 * n, height - 2.0 * n);
                pageFormat2.setPaper(paper);
                return pageFormat2;
            }
        }
        String country = Locale.getDefault().getCountry();
        if (!Locale.getDefault().equals(Locale.ENGLISH) && country != null && !country.equals(Locale.US.getCountry()) && !country.equals(Locale.CANADA.getCountry())) {
            double n2 = 25.4;
            double rint = Math.rint(210.0 * n / n2);
            double rint2 = Math.rint(297.0 * n / n2);
            paper.setSize(rint, rint2);
            paper.setImageableArea(n, n, rint - 2.0 * n, rint2 - 2.0 * n);
        }
        pageFormat2.setPaper(paper);
        return pageFormat2;
    }
    
    @Override
    public PageFormat validatePage(PageFormat pageFormat) {
        PageFormat pageFormat2 = (PageFormat)pageFormat.clone();
        Paper paper = new Paper();
        this.validatePaper(pageFormat2.getPaper(), paper);
        pageFormat2.setPaper(paper);
        return pageFormat2;
    }
    
    @Override
    public void setCopies(int mNumCopies) {
        this.mNumCopies = mNumCopies;
    }
    
    @Override
    public int getCopies() {
        return this.mNumCopies;
    }
    
    protected int getCopiesInt() {
        return (this.copiesAttr > 0) ? this.copiesAttr : this.getCopies();
    }
    
    @Override
    public String getUserName() {
        return System.getProperty("user.name");
    }
    
    protected String getUserNameInt() {
        if (this.userNameAttr != null) {
            return this.userNameAttr;
        }
        try {
            return this.getUserName();
        }
        catch (SecurityException ex) {
            return "";
        }
    }
    
    @Override
    public void setJobName(String mDocName) {
        if (mDocName != null) {
            this.mDocName = mDocName;
            return;
        }
        throw new NullPointerException();
    }
    
    @Override
    public String getJobName() {
        return this.mDocName;
    }
    
    protected String getJobNameInt() {
        return (this.jobNameAttr != null) ? this.jobNameAttr : this.getJobName();
    }
    
    protected void setPageRange(int mFirstPage, int mLastPage) {
        if (mFirstPage >= 0 && mLastPage >= 0) {
            this.mFirstPage = mFirstPage;
            this.mLastPage = mLastPage;
            if (this.mLastPage < this.mFirstPage) {
                this.mLastPage = this.mFirstPage;
            }
        }
        else {
            this.mFirstPage = -1;
            this.mLastPage = -1;
        }
    }
    
    protected int getFirstPage() {
        return (this.mFirstPage == -1) ? 0 : this.mFirstPage;
    }
    
    protected int getLastPage() {
        return this.mLastPage;
    }
    
    protected void setCollated(boolean mCollate) {
        this.mCollate = mCollate;
        this.collateAttReq = true;
    }
    
    protected boolean isCollated() {
        return this.mCollate;
    }
    
    protected final int getSelectAttrib() {
        if (this.attributes != null) {
            SunPageSelection sunPageSelection = (SunPageSelection)this.attributes.get(SunPageSelection.class);
            if (sunPageSelection == SunPageSelection.RANGE) {
                return 2;
            }
            if (sunPageSelection == SunPageSelection.SELECTION) {
                return 1;
            }
            if (sunPageSelection == SunPageSelection.ALL) {
                return 0;
            }
        }
        return 4;
    }
    
    protected final int getFromPageAttrib() {
        if (this.attributes != null) {
            PageRanges pageRanges = (PageRanges)this.attributes.get(PageRanges.class);
            if (pageRanges != null) {
                return pageRanges.getMembers()[0][0];
            }
        }
        return this.getMinPageAttrib();
    }
    
    protected final int getToPageAttrib() {
        if (this.attributes != null) {
            PageRanges pageRanges = (PageRanges)this.attributes.get(PageRanges.class);
            if (pageRanges != null) {
                int[][] members = pageRanges.getMembers();
                return members[members.length - 1][1];
            }
        }
        return this.getMaxPageAttrib();
    }
    
    protected final int getMinPageAttrib() {
        if (this.attributes != null) {
            SunMinMaxPage sunMinMaxPage = (SunMinMaxPage)this.attributes.get(SunMinMaxPage.class);
            if (sunMinMaxPage != null) {
                return sunMinMaxPage.getMin();
            }
        }
        return 1;
    }
    
    protected final int getMaxPageAttrib() {
        if (this.attributes != null) {
            SunMinMaxPage sunMinMaxPage = (SunMinMaxPage)this.attributes.get(SunMinMaxPage.class);
            if (sunMinMaxPage != null) {
                return sunMinMaxPage.getMax();
            }
        }
        Pageable pageable = this.getPageable();
        if (pageable != null) {
            int numberOfPages = pageable.getNumberOfPages();
            if (numberOfPages <= -1) {
                numberOfPages = 9999;
            }
            return (numberOfPages == 0) ? 1 : numberOfPages;
        }
        else {
            return Integer.MAX_VALUE;
        }
    }
    
    protected abstract void startDoc() throws PrinterException;
    
    protected abstract void endDoc() throws PrinterException;
    
    protected abstract void abortDoc();
    
    protected void cancelDoc() throws PrinterAbortException {
        this.abortDoc();
        synchronized (this) {
            this.userCancelled = false;
            this.performingPrinting = false;
            this.notify();
        }
        throw new PrinterAbortException();
    }
    
    protected int getCollatedCopies() {
        return this.isCollated() ? this.getCopiesInt() : 1;
    }
    
    protected int getNoncollatedCopies() {
        return this.isCollated() ? 1 : this.getCopiesInt();
    }
    
    synchronized void setGraphicsConfigInfo(AffineTransform defaultDeviceTransform, double x, double y) {
        Point2D.Double double1 = new Point2D.Double(x, y);
        defaultDeviceTransform.transform(double1, double1);
        if (this.pgConfig == null || this.defaultDeviceTransform == null || !defaultDeviceTransform.equals(this.defaultDeviceTransform) || this.deviceWidth != (int)double1.getX() || this.deviceHeight != (int)double1.getY()) {
            this.deviceWidth = (int)double1.getX();
            this.deviceHeight = (int)double1.getY();
            this.defaultDeviceTransform = defaultDeviceTransform;
            this.pgConfig = null;
        }
    }
    
    synchronized PrinterGraphicsConfig getPrinterGraphicsConfig() {
        if (this.pgConfig != null) {
            return this.pgConfig;
        }
        String string = "Printer Device";
        PrintService printService = this.getPrintService();
        if (printService != null) {
            string = printService.toString();
        }
        return this.pgConfig = new PrinterGraphicsConfig(string, this.defaultDeviceTransform, this.deviceWidth, this.deviceHeight);
    }
    
    protected int printPage(Pageable pageable, int n) throws PrinterException {
        PageFormat pageFormat;
        PageFormat pageFormat2;
        Printable printable;
        try {
            pageFormat = pageable.getPageFormat(n);
            pageFormat2 = (PageFormat)pageFormat.clone();
            printable = pageable.getPrintable(n);
        }
        catch (Exception ex) {
            PrinterException ex2 = new PrinterException("Error getting page or printable.[ " + ex + " ]");
            ex2.initCause(ex);
            throw ex2;
        }
        Paper paper = pageFormat2.getPaper();
        if (pageFormat2.getOrientation() != 1 && this.landscapeRotates270) {
            double imageableX = paper.getImageableX();
            double imageableY = paper.getImageableY();
            double imageableWidth = paper.getImageableWidth();
            double imageableHeight = paper.getImageableHeight();
            paper.setImageableArea(paper.getWidth() - imageableX - imageableWidth, paper.getHeight() - imageableY - imageableHeight, imageableWidth, imageableHeight);
            pageFormat2.setPaper(paper);
            if (pageFormat2.getOrientation() == 0) {
                pageFormat2.setOrientation(2);
            }
            else {
                pageFormat2.setOrientation(0);
            }
        }
        double sx = this.getXRes() / 72.0;
        double sy = this.getYRes() / 72.0;
        Rectangle2D.Double o = new Rectangle2D.Double(paper.getImageableX() * sx, paper.getImageableY() * sy, paper.getImageableWidth() * sx, paper.getImageableHeight() * sy);
        AffineTransform transform = new AffineTransform();
        AffineTransform o2 = new AffineTransform();
        o2.scale(sx, sy);
        int n2 = (int)o.getWidth();
        if (n2 % 4 != 0) {
            n2 += 4 - n2 % 4;
        }
        if (n2 <= 0) {
            throw new PrinterException("Paper's imageable width is too small.");
        }
        int n3 = (int)o.getHeight();
        if (n3 <= 0) {
            throw new PrinterException("Paper's imageable height is too small.");
        }
        int n4 = 4194304 / n2 / 3;
        int n5 = (int)Math.rint(paper.getImageableX() * sx);
        int n6 = (int)Math.rint(paper.getImageableY() * sy);
        AffineTransform o3 = new AffineTransform();
        o3.translate((double)(-n5), (double)n6);
        o3.translate(0.0, (double)n4);
        o3.scale(1.0, -1.0);
        PeekGraphics peekGraphics = this.createPeekGraphics(new BufferedImage(1, 1, 5).createGraphics(), this);
        Rectangle2D.Double o4 = new Rectangle2D.Double(pageFormat2.getImageableX(), pageFormat2.getImageableY(), pageFormat2.getImageableWidth(), pageFormat2.getImageableHeight());
        peekGraphics.transform(o2);
        peekGraphics.translate(-this.getPhysicalPrintableX(paper) / sx, -this.getPhysicalPrintableY(paper) / sy);
        peekGraphics.transform(new AffineTransform(pageFormat2.getMatrix()));
        this.initPrinterGraphics(peekGraphics, o4);
        AffineTransform transform2 = peekGraphics.getTransform();
        this.setGraphicsConfigInfo(o2, paper.getWidth(), paper.getHeight());
        int print = printable.print(peekGraphics, pageFormat, n);
        this.debug_println("pageResult " + print);
        if (print == 0) {
            this.debug_println("startPage " + n);
            Paper paper2 = pageFormat2.getPaper();
            int n7 = this.previousPaper == null || paper2.getWidth() != this.previousPaper.getWidth() || paper2.getHeight() != this.previousPaper.getHeight();
            this.previousPaper = paper2;
            this.startPage(pageFormat2, printable, n, (boolean)n7);
            Graphics2D pathGraphics = this.createPathGraphics(peekGraphics, this, printable, pageFormat2, n);
            if (pathGraphics != null) {
                pathGraphics.transform(o2);
                pathGraphics.translate(-this.getPhysicalPrintableX(paper) / sx, -this.getPhysicalPrintableY(paper) / sy);
                pathGraphics.transform(new AffineTransform(pageFormat2.getMatrix()));
                this.initPrinterGraphics(pathGraphics, o4);
                this.redrawList.clear();
                AffineTransform transform3 = pathGraphics.getTransform();
                printable.print(pathGraphics, pageFormat, n);
                for (int j = 0; j < this.redrawList.size(); ++j) {
                    RasterPrinterJob.GraphicsState graphicsState = (RasterPrinterJob.GraphicsState)this.redrawList.get(j);
                    pathGraphics.setTransform(transform3);
                    ((PathGraphics)pathGraphics).redrawRegion(graphicsState.region, graphicsState.sx, graphicsState.sy, graphicsState.theClip, graphicsState.theTransform);
                }
            }
            else {
                BufferedImage cachedBand = this.cachedBand;
                if (this.cachedBand == null || n2 != this.cachedBandWidth || n4 != this.cachedBandHeight) {
                    cachedBand = new BufferedImage(n2, n4, 5);
                    this.cachedBand = cachedBand;
                    this.cachedBandWidth = n2;
                    this.cachedBandHeight = n4;
                }
                Graphics2D graphics = cachedBand.createGraphics();
                this.initPrinterGraphics(graphics, new Rectangle2D.Double(0.0, 0.0, (double)n2, (double)n4));
                ProxyGraphics2D o5 = new ProxyGraphics2D(graphics, this);
                Graphics2D graphics2 = cachedBand.createGraphics();
                graphics2.setColor(Color.white);
                byte[] dataStorage = ((ByteInterleavedRaster)cachedBand.getRaster()).getDataStorage();
                int n8 = (int)this.getPhysicalPrintableX(paper);
                int n9 = (int)this.getPhysicalPrintableY(paper);
                for (int i = 0; i <= n3; i += n4) {
                    graphics2.fillRect(0, 0, n2, n4);
                    graphics.setTransform(transform);
                    graphics.transform(o3);
                    o3.translate(0.0, (double)(-n4));
                    graphics.transform(o2);
                    graphics.transform(new AffineTransform(pageFormat2.getMatrix()));
                    Rectangle bounds = transform2.createTransformedShape(graphics.getClipBounds()).getBounds();
                    if (bounds == null || (peekGraphics.hitsDrawingArea(bounds) && n2 > 0 && n4 > 0)) {
                        int n10 = n5 - n8;
                        if (n10 < 0) {
                            graphics.translate((double)n10 / sx, 0.0);
                            n10 = 0;
                        }
                        int n11 = n6 + i - n9;
                        if (n11 < 0) {
                            graphics.translate(0.0, (double)n11 / sy);
                            n11 = 0;
                        }
                        o5.setDelegate((Graphics2D)graphics.create());
                        printable.print(o5, pageFormat, n);
                        o5.dispose();
                        this.printBand(dataStorage, n10, n11, n2, n4);
                    }
                }
                graphics2.dispose();
                graphics.dispose();
            }
            this.debug_println("calling endPage " + n);
            this.endPage(pageFormat2, printable, n);
        }
        return print;
    }
    
    @Override
    public void cancel() {
        synchronized (this) {
            if (this.performingPrinting) {
                this.userCancelled = true;
            }
            this.notify();
        }
    }
    
    @Override
    public boolean isCancelled() {
        boolean n = false;
        synchronized (this) {
            n = (this.performingPrinting && this.userCancelled);
            this.notify();
        }
        return n;
    }
    
    protected Pageable getPageable() {
        return this.mDocument;
    }
    
    protected Graphics2D createPathGraphics(PeekGraphics peekGraphics, PrinterJob printerJob, Printable printable, PageFormat pageFormat, int n) {
        return null;
    }
    
    protected PeekGraphics createPeekGraphics(Graphics2D graphics2D, PrinterJob printerJob) {
        return new PeekGraphics(graphics2D, printerJob);
    }
    
    protected void initPrinterGraphics(Graphics2D graphics2D, Rectangle2D clip) {
        graphics2D.setClip(clip);
        graphics2D.setPaint(Color.black);
    }
    
    public boolean checkAllowedToPrintToFile() {
        try {
            this.throwPrintToFile();
            return true;
        }
        catch (SecurityException ex) {
            return false;
        }
    }
    
    private void throwPrintToFile() {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            if (this.printToFilePermission == null) {
                this.printToFilePermission = new FilePermission("<<ALL FILES>>", "read,write");
            }
            securityManager.checkPermission(this.printToFilePermission);
        }
    }
    
    protected String removeControlChars(String s) {
        char[] charArray = s.toCharArray();
        int length = charArray.length;
        char[] value = new char[length];
        int count = 0;
        for (int n : charArray) {
            if (n > '\r' || n < '\t' || n == '\u000b' || n == '\f') {
                value[count++] = n;
            }
        }
        if (count == length) {
            return s;
        }
        return new String(value, 0, count);
    }
    
    private long getParentWindowID() {
        return this.parentWindowID;
    }
    
    private void clearParentWindowID() {
        this.parentWindowID = 0L;
        this.onTop = null;
    }
    
    private void setParentWindowID(PrintRequestAttributeSet set) {
        this.parentWindowID = 0L;
        this.onTop = (DialogOnTop)set.get(DialogOnTop.class);
        if (this.onTop != null) {
            this.parentWindowID = this.onTop.getID();
        }
    }
    
    static {
        RasterPrinterJob.forcePDL = false;
        RasterPrinterJob.forceRaster = false;
        RasterPrinterJob.shapeTextProp = false;
        String s = (String)AccessController.doPrivileged(new GetPropertyAction("sun.java2d.print.pipeline"));
        if (s != null) {
            if (s.equalsIgnoreCase("pdl")) {
                RasterPrinterJob.forcePDL = true;
            }
            else if (s.equalsIgnoreCase("raster")) {
                RasterPrinterJob.forceRaster = true;
            }
        }
        if ((String)AccessController.doPrivileged(new GetPropertyAction("sun.java2d.print.shapetext")) != null) {
            RasterPrinterJob.shapeTextProp = true;
        }
        RasterPrinterJob.debugPrint = false;
    }
    
    private class GraphicsState
    {
        Rectangle2D region;
        Shape theClip;
        AffineTransform theTransform;
        double sx;
        double sy;
    }
}
